import os
import re
import logging


class StreamHandler(logging.StreamHandler):

    def __init__(self):
        logging.StreamHandler.__init__(self)
        fmt = '%(asctime)s %(filename)s %(levelname)s: %(message)s'
        fmt_date = '%Y-%m-%dT%T%Z'
        formatter = logging.Formatter(fmt, fmt_date)
        self.setFormatter(formatter)


class FileHandler(logging.FileHandler):

    def __init__(self, path):
        abspath = os.path.abspath(path)
        abspath_dir = re.search("^.+\/", abspath).group(0)
        if not os.path.exists(abspath_dir):
            os.makedirs(abspath_dir)
        if os.path.isdir(abspath):
            abspath = os.path.join(abspath, re.sub("py", "log", __file__))
        logging.FileHandler.__init__(self, abspath)
        fmt = '%(asctime)s %(filename)s %(levelname)s: %(message)s'
        fmt_date = '%Y-%m-%dT%T%Z'
        formatter = logging.Formatter(fmt, fmt_date)
        self.setFormatter(formatter)



import logger
import lvm
from prometheus_client import Gauge
import re
import traceback
import time

import logging


def loop(func):
    def wrapper(self, *args, **kwargs):
        if self.timeout is not None:
            while True:
                func(self, *args, **kwargs)
                time.sleep(self.timeout)
        else:
            func(self, *args, **kwargs)
        return
    return wrapper


class LVM(object):
    def __init__(self, timeout=None, loglevel=None, filelog=None):
        global log

        if "log" not in globals():
            if filelog is not None:
                log = logging.getLogger(__name__)
                log.addHandler(logger.FileHandler(filelog))
                log.setLevel(getattr(logging, loglevel))
            else:
                log = logging.getLogger(__name__)
                log.addHandler(logger.StreamHandler())
                log.setLevel(getattr(logging, loglevel))

        self.timeout = timeout
        self.metric = Gauge("lvm", "LVM data in percentages", ["lv", "vg"])

    @loop
    def collect(self):
        try:
            vg_names = lvm.listVgNames()
            for vg_name in vg_names:
                vg = lvm.vgOpen(vg_name, 'r')
                lvs = vg.listLVs()
                for lv in lvs:
                    self.metric.labels(lv.getName(), vg_name).set(
                        lv.getProperty("data_percent")[0] / 1000000.0)
        except:
            log.error("Something wrong with lvm data collection\n{}".
                format(traceback.format_exc()))



import argparse
import logger
from prometheus_client import start_http_server

import logging
from lvm_collector import LVM


def main():
    try:
        parser = argparse.ArgumentParser()

        parser.add_argument("-c", "--collectors", dest="collectors",
            nargs='+', default=[], choices=["lvm"],
            help="List of desired collectors to include")
        parser.add_argument("-p", "--port", dest="port",
            default=8000, help="Port of http info server")
        parser.add_argument("-t", "--timeout", dest="timeout",
            default=10, help="Timeout of cleaning. "
                "Live it empty in case of using cron job.")
        parser.add_argument("--loglevel", dest="loglevel",
            default="INFO",
            choices=["CRITICAL", "ERROR", "WARNING",
                "INFO", "DEBUG", "NOTSET"],
            help="Logging level")
        parser.add_argument("-l", "--log", dest="log",
            help="Redirect logging to file")
        args = parser.parse_args()

        global log

        if args.log is not None:
            log = logging.getLogger(__name__)
            log.addHandler(logger.FileHandler(args.log))
            log.setLevel(getattr(logging, args.loglevel))
        else:
            log = logging.getLogger(__name__)
            log.addHandler(logger.StreamHandler())
            log.setLevel(getattr(logging, args.loglevel))

        start_http_server(int(args.port))

        if "lvm" in args.collectors:
            lvm_collector = LVM(int(args.timeout), args.loglevel, args.log)
            lvm_collector.collect()

    except KeyboardInterrupt:
        print('\nThe process was interrupted by the user')
        raise SystemExit

if __name__ == "__main__":
    main()
